package iotcloud;

import java.util.Set;
import java.util.Map;
import java.util.HashSet;

import javax.script.ScriptException;
import java.lang.NullPointerException;


class PendingTransaction {

    private Set<KeyValue> keyValueUpdateSet = null;
    private Set<KeyValue> keyValueGuardSet = null;
    private long arbitrator = -1;

    public PendingTransaction() {
        keyValueUpdateSet = new HashSet<KeyValue>();
        keyValueGuardSet = new HashSet<KeyValue>();
    }

    /**
     * Add a new key value to the updates
     *
     */
    public void addKV(KeyValue newKV) {

        KeyValue rmKV = null;

        // Make sure there are no duplicates
        for (KeyValue kv : keyValueUpdateSet) {
            if (kv.getKey().equals(newKV.getKey())) {

                // Remove key if we are adding a newer version of the same key
                rmKV = kv;
                break;
            }
        }

        // Remove key if we are adding a newer version of the same key
        if (rmKV != null) {
            keyValueUpdateSet.remove(rmKV);
        }

        // Add the key to the hash set
        keyValueUpdateSet.add(newKV);
    }


    /**
     * Add a new key value to the guard set
     *
     */
    public void addKVGuard(KeyValue newKV) {
        // Add the key to the hash set
        keyValueGuardSet.add(newKV);
    }

    /**
     * Checks if the arbitrator is the same
     *
     */
    public boolean checkArbitrator(long arb) {
        if (arbitrator == -1) {
            arbitrator = arb;
            return true;
        }

        return arb == arbitrator;
    }

    /**
     * Get the transaction arbitrator
     *
     */
    public long getArbitrator() {
        return arbitrator;
    }

    /**
     * Get the key value update set
     *
     */
    public Set<KeyValue> getKVUpdates() {
        return keyValueUpdateSet;
    }


    /**
       * Get the key value update set
       *
       */
    public Set<KeyValue> getKVGuard() {
        return keyValueGuardSet;
    }

    public boolean evaluateGuard(Map<IoTString, KeyValue> keyValTableCommitted, Map<IoTString, KeyValue> keyValTableSpeculative, Map<IoTString, KeyValue> keyValTablePendingTransSpeculative) {
        for (KeyValue kvGuard : keyValueGuardSet) {

            // First check if the key is in the speculative table, this is the value of the latest assumption
            KeyValue kv = keyValTablePendingTransSpeculative.get(kvGuard.getKey());


            if (kv == null) {
                // if it is not in the pending trans table then check the speculative table and use that
                // value as our latest assumption
                kv = keyValTableSpeculative.get(kvGuard.getKey());
            }


            if (kv == null) {
                // if it is not in the speculative table then check the committed table and use that
                // value as our latest assumption
                kv = keyValTableCommitted.get(kvGuard.getKey());
            }

            if (kvGuard.getValue() != null) {
                if ((kv == null) || (!kvGuard.getValue().equals(kv.getValue()))) {
                    return false;
                }
            } else {
                if (kv != null) {
                    return false;
                }
            }
        }
        return true;
    }
}